/* ----------------------------------------------------------------------
 * commands.c
 *
 * MILO debugger/monitor commands
 * Written by Andreas Busse
 *
 * This file is subject to the terms and conditions of the
 * GNU General Public License. See the file COPYING in the
 * main directory of this archive for more details.
 * ---------------------------------------------------------------------- */

#include <stand/debug.h>

#ifdef STANDALONE
#include <stdio.h>
#else
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/file.h>
#include <sys/types.h>
#include <arc/arc.h>
#endif

/* local prototypes */

unsigned get_memory(unsigned caddr, unsigned width);
char *atob(char *cp, unsigned *iptr);
void showchar(unsigned c);

/* ----------------------------------------------------------------------
 * dump()
 * dump memory
 * ---------------------------------------------------------------------- */
dump(argc, argv)
     int argc;
     char **argv;
{
  unsigned address1 = 0x0;
  unsigned address2 = 0x0;
  unsigned val;
  int width = SW_WORD;
  
  argv++;
  argc--;
#if DEBUG  
  printf("argc=%d, argv[0]=%s\r\n",argc,argv[0]);
#endif  
  if (argc >= 1 && argv[0][0] == '-')
    {
      switch (argv[0][1])
	{
	case 'h':
	  width = SW_HALFWORD;
	  break;
	case 'w':
	  width = SW_WORD;
	  break;
	case 'b':
	  width = SW_BYTE;
	  break;
	default:
	  fprintf("Invalid size: %s\r\n",argv[0]);
	  return(1);
	}
      argv++;
      argc--;
    }

#if DEBUG  
  printf("argc=%d, argv[0]=%s\r\n",argc,argv[0]);
#endif  
  if (!argc)
    {
      fprintf(stderr,"Missing argument(s)\r\n");
      return(1);
    }

  /* must be an address, get start address */
      
  if (*atob(*argv, &address1))
    {
      fprintf(stderr,"Illegal address: %s\n\r", *argv);
      return(1);
    }
  argc--;
  argv++;

  if (argc)			/* more arguments, get end address */
    {
      if (*atob(*argv, &address2))
	{
	  fprintf(stderr,"Illegal address: %s\n\r", *argv);
	  return(1);
	}
      argc--;
      argv++;
    }

  if (argc)
    {
      fprintf(stderr,"Illegal argument: %s\n\r", *argv);
      return(1);
    }
      
  if (!address2)		/* end address not given, so we assume +0x100 */
    address2 = address1 + 0x100;
  if (address2 <= address1)
    {
      fprintf(stderr,"Illegal address range\r\n");
      return(1);
    }

#if DEBUG
  printf("address1 = 0x%08x\r\n",address1);
  printf("address2 = 0x%08x\r\n",address2);
#endif
  
  do_dump(address1,address2,width);
  return(0);
}

/* ----------------------------------------------------------------------
 * do_dump() -- Readout memory and display
 * ---------------------------------------------------------------------- */
int do_dump(address1,address2,width)
     unsigned address1;
     unsigned address2;
     int width;
{
  unsigned val;
  char c;
  int i;
  
  address1 &= ~(width-1);
  address2 &= ~(width-1);
  address2 += width;
  
  while(address1 < address2)
    {
      printf("0x%08x:  ",address1);
      for (i=0; i<16; i+=width)
	{
	  val = get_memory(address1+i,width);
	  switch(width)
	    {
	    case SW_BYTE:
	      printf("%02x ",val);
	      break;
	    case SW_HALFWORD:
	      printf("%04x ",val);
	      break;
	    case SW_WORD:
	      printf("%08x ",val);
	      break;
	    }
	}
      printf("  ");
      for (i=0; i<16; i++)
	{
	  c = get_memory(address1+i,1);
	  showchar(c);
	}

      printf("\r\n");
      address1 += i;
    }
}

/* ----------------------------------------------------------------------
 * get_memory()
 * ---------------------------------------------------------------------- */
unsigned get_memory(caddr, width)
     unsigned caddr;
     unsigned width;
{
  switch (width)
    {
    case SW_BYTE:
      return (*(unsigned char *)caddr);
    case SW_HALFWORD:
      return (*(unsigned short *)caddr);
    case SW_WORD:
      return(*(unsigned *)caddr);
    default:
      printf("get_memory: Illegal width\r\n");
    }
}

/* ----------------------------------------------------------------------
 * showchar()
 * ---------------------------------------------------------------------- */
void showchar(c)
     unsigned c;
{
  if (c >= 32 && c <= 127)
    printf("%c",c);
  else
    printf("_");
}

/* ----------------------------------------------------------------------
 * fill() -- Fill memory 
 * ---------------------------------------------------------------------- */
int fill(argc, argv)
     int argc;
     char **argv;
{
  unsigned address1;
  unsigned address2;
  unsigned val;
  int width = SW_WORD;

  argv++;
  argc--;
  if (argc >= 1 && argv[0][0] == '-')
    {
      switch (argv[0][1])
	{
	case 'h':
	  width = SW_HALFWORD;
	  break;
	case 'w':
	  width = SW_WORD;
	  break;
	case 'b':
	  width = SW_BYTE;
	  break;
	default:
	  fprintf("Invalid size: %s\r\n",argv[0]);
	  return(1);
	}
      argv++;
      argc--;
    }

  if (argc < 2)
    {
      fprintf(stderr,"Missing argument(s)\r\n");
      return(1);
    }
      
  /* must be an address, get start address */
      
  if (*atob(*argv, &address1))
    {
      fprintf(stderr,"Illegal address: %s\n\r", *argv);
      return(1);
    }
  argc--;
  argv++;

  if (argc > 1)			/* more arguments, get end address */
    {
      if (*atob(*argv, &address2))
	{
	  fprintf(stderr,"Illegal address: %s\n\r", *argv);
	  return(1);
	}
      argc--;
      argv++;
    }

     
  if (!address2)		/* end address not given, so we assume +width */
    address2 = address1 + width;
  if (address2 <= address1)
    {
      fprintf(stderr,"Illegal address range\r\n");
      return(1);
    }

#if DEBUG  
  printf("address1 = 0x%08x\r\n",address1);
  printf("address2 = 0x%08x\r\n",address2);
#endif
  
  if (argc > 1)
    {
      fprintf(stderr,"Illegal argument: %s\n\r", *argv[1]);
      return(1);
    }
  else if (!argc)
    {
      fprintf(stderr,"Missing argument\r\n");
      return(1);
    }
	
  if (*atob(*argv, &val))
    {
      fprintf(stderr,"Invalid argument: %s\r\n", *argv);
      return(1);
    }
#if DEBUG
  printf("val = 0x%08x\r\n",val);
#endif
  
  do_fill(address1,address2,width,val);
  return(0);
}

/* ----------------------------------------------------------------------
 * do_fill()
 * ---------------------------------------------------------------------- */
int do_fill(address1,address2,width,val)
     unsigned address1;
     unsigned address2;
     int width;
     unsigned val;
{
  address1 &= ~(width-1);
  address2 &= ~(width-1);
  address2 += width;
  
  while(address1 < address2)
    {
      switch(width)
	{
	case SW_BYTE:
	  *((char *)address1) = val;
	  break;
	case SW_HALFWORD:
	  *((short *)address1) = val;
	  break;
	case SW_WORD:
	  *((int *)address1) = val;
	  break;
	}
      address1 += width;
    }
  return(0);
}

/* end of file commands.c */
